package de.hft.stuttgart.citydoctor2.datastructure;

import de.hft.stuttgart.citydoctor2.check.Check;
import de.hft.stuttgart.citydoctor2.check.CheckError;
import de.hft.stuttgart.citydoctor2.check.CheckId;
import de.hft.stuttgart.citydoctor2.parser.ParserConfiguration;
import de.hft.stuttgart.citydoctor2.utils.CopyHandler;
import de.hft.stuttgart.citydoctor2.utils.Copyable;
import org.citygml4j.core.util.geometry.GeometryFactory;

import java.io.Serial;
import java.util.ArrayList;
import java.util.List;

public class Storey extends AbstractBuildingSubdivision {

    @Serial
    private static final long serialVersionUID = 5162814691642668089L;

    private final List<BuildingUnit> buildingUnits = new ArrayList<>();

    @Override
    public Copyable createCopyInstance() {
        return new Storey();
    }

    @Override
    public void unsetGmlGeometries() {
        super.unsetGmlGeometries();
        for (BuildingUnit buildingUnit : buildingUnits) {
            buildingUnit.unsetGmlGeometries();
        }
    }

    @Override
    public void reCreateGeometries(GeometryFactory factory, ParserConfiguration config) {
        super.reCreateGeometries(factory, config);
        for (BuildingUnit buildingUnit : buildingUnits) {
            buildingUnit.reCreateGeometries(factory, config);
        }
    }

    @Override
    public void accept(Check c) {
        super.accept(c);
        for (BuildingUnit buildingUnit : buildingUnits) {
            buildingUnit.accept(c);
        }
    }

    @Override
    public void collectContainedErrors(List<CheckError> errors) {
        super.collectContainedErrors(errors);
        for (BuildingUnit buildingUnit : buildingUnits) {
            buildingUnit.collectContainedErrors(errors);
        }
    }

    @Override
    public void clearAllContainedCheckResults() {
        super.clearAllContainedCheckResults();
        for (BuildingUnit buildingUnit : buildingUnits) {
            buildingUnit.clearAllContainedCheckResults();
        }
    }

    @Override
    public boolean containsError(CheckId checkIdentifier) {
        boolean hasError = super.containsError(checkIdentifier);
        if (hasError) {
            return true;
        }
        for (BuildingUnit buildingUnit : buildingUnits) {
            if (buildingUnit.containsError(checkIdentifier)) {
                return true;
            }
        }
        return false;
    }

    @Override
    public boolean containsAnyError() {
        boolean hasError = super.containsAnyError();
        if (hasError) {
            return true;
        }
        for (BuildingUnit buildingUnit : buildingUnits) {
            if (buildingUnit.containsAnyError()) {
                return true;
            }
        }
        return false;
    }

    @Override
    public void prepareForChecking() {
        super.prepareForChecking();
        for (BuildingUnit buildingUnit : buildingUnits) {
            buildingUnit.prepareForChecking();
        }
    }

    @Override
    public void clearMetaInformation() {
        super.clearMetaInformation();
        for (BuildingUnit buildingUnit : buildingUnits) {
            buildingUnit.clearMetaInformation();
        }
    }

    @Override
    public void collectInstances(CopyHandler handler) {
        super.collectInstances(handler);
        for (BuildingUnit buildingUnit : buildingUnits) {
            buildingUnit.collectInstances(handler);
        }
    }

    @Override
    public void fillValues(Copyable original, CopyHandler handler) {
        super.fillValues(original, handler);
        Storey originalSt = (Storey) original;
        for (BuildingUnit buildingUnit : originalSt.buildingUnits) {
            buildingUnits.add(handler.getCopyInstance(buildingUnit));
        }
        this.abs = originalSt.abs;
    }

    public List<BuildingUnit> getBuildingUnits() {
        return buildingUnits;
    }
}
